﻿using Molport.API;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace MolPortAPIExampleConsole
{
    class Program
    {
        private const long RESULT__SUCCESS__ID = 1;
        private const long RESULT__UNKNOWN_ERROR__ID = 2;
        private const long RESULT__MOLECULE_WAS_NOT_FOUND__ID = 3;
        private const long RESULT__USER_IS_NOT_AUTHORIZED__ID = 4;
        private const long RESULT__GRANTED_REQUEST_COUNT_EXCEEDED__ID = 5;

        static void Main(string[] args)
        {
            // user name provided by MolPort
            String userName = null;

            // user password provided by MolPort
            String userPassword = null;

            // MolPort molecule id
            String moleculeId = null;

            if (args == null || args.Length != 3)
            {
                Console.Out.WriteLine("Please, provide: first argument \"username\", second argument - \"password\", third argument - MolPort molecule ID!");
                Console.Read();
                return;
            }
            else
            {
                userName = args[0];
                userPassword = args[1];
                moleculeId = args[2];
            }

            try
            {

                // test URL to extract information about 
                var uri = string.Format("https://www.molport.com/api/molecule/search/molport-id/{0}", moleculeId);

                // create web request
                var webRequest = (HttpWebRequest)WebRequest.Create(uri);

                //
                // Authorization
                //

                webRequest.Credentials = new NetworkCredential(userName, userPassword);


                //
                // Execute request and extract response data
                //

                // create web response
                var webResponse = (HttpWebResponse)webRequest.GetResponse();

                // extract XML
                var xml = new StreamReader(webResponse.GetResponseStream()).ReadToEnd();


                //
                // Deserialize MolPort XML
                //

                var molecule = Molport.API.data.Deserialize(xml);


                //
                // Right now "molecule" object contains deserialized MolPort molecule XML data
                //

                // checks returned status for "SUCCESS" (1)
                if (molecule.result == RESULT__SUCCESS__ID)
                {
                    // basic data
                    Console.Out.WriteLine(string.Format("Molecule ID: {0}", molecule.molecule.moleculeid));
                    Console.Out.WriteLine(string.Format("MolWeight: {0}", molecule.molecule.molweight));
                    Console.Out.WriteLine(string.Format("Formula: {0}", molecule.molecule.formula));
                    Console.Out.WriteLine(string.Format("ChemAxon SMILES: {0}", molecule.molecule.chemaxonsmiles));
                    Console.Out.WriteLine(string.Format("IUPAC: {0}", molecule.molecule.iupac));
                    Console.Out.WriteLine(string.Format("Standard InChi key:  {0}", molecule.molecule.standardinchikey));

                    // synonyms
                    if (molecule.molecule.synonyms != null && molecule.molecule.synonyms.Count > 0)
                        foreach (String synonym in molecule.molecule.synonyms)
                            Console.Out.WriteLine("Synonym: " + synonym);

                    // supplier sections
                    if (molecule.molecule.suppliers != null && molecule.molecule.suppliers.Count > 0)
                    {
                        foreach (dataMoleculeSection section in molecule.molecule.suppliers)
                        {
                            Console.Out.WriteLine("Type: " + section.type);

                            if (section.supplier != null && section.supplier.Count > 0)
                                foreach (dataMoleculeSectionSupplier supplier in section.supplier)
                                    Console.Out.WriteLine("\tSupplier name: " + supplier.name);

                            //
                            // ... read other data ...
                            //
                        }
                    }
                }
                else if (molecule.result == RESULT__UNKNOWN_ERROR__ID)
                    Console.Out.WriteLine("Unknown error! Please, contact MolPort administrator!");
                else if (molecule.result == RESULT__MOLECULE_WAS_NOT_FOUND__ID)
                    Console.Out.WriteLine("Molecule was not found!");
                else if (molecule.result == RESULT__USER_IS_NOT_AUTHORIZED__ID)
                    Console.Out.WriteLine("User is not authorized! Please, provide correct username and password provided by MolPort!");
                else if (molecule.result == RESULT__GRANTED_REQUEST_COUNT_EXCEEDED__ID)
                    Console.Out.WriteLine("Request count exceeded! Please, contact MolPort to increase the count!");

                Console.Read();
            }
            catch (Exception exception)
            {
                Console.Out.WriteLine("ERROR : " + exception.Message);
            }
        }
    }
}
